{
    XviD Stats Analyzer
    Copyright (C) 2005 Yuri Khan
    stats-analyzer at gorodok dot net

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

unit GraphUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Contnrs, StdCtrls, Registry, ComCtrls, ColorsUnit;

type
  TFrameInfo = class
  private
    FFrameType: Char;
    FQuantizer: Integer;
    FIntraBlocks: Integer;
    FFrameSize: Integer;
    FInterBlocks: Integer;
    FHeaderSize: Integer;
    FNotCodedBlocks: Integer;
    FFilteredFrameSize: Integer;
  public
    property FrameType: Char read FFrameType;
    property Quantizer: Integer read FQuantizer;
    property IntraBlocks: Integer read FIntraBlocks;
    property InterBlocks: Integer read FInterBlocks;
    property NotCodedBlocks: Integer read FNotCodedBlocks;
    property FrameSize: Integer read FFrameSize;
    property HeaderSize: Integer read FHeaderSize;

    procedure Read(var f: TextFile);
    property FilteredFrameSize: Integer read FFilteredFrameSize
      write FFilteredFrameSize;
  end;

  TPaintScrollBox = class(TScrollBox)
  private
    FOnPaint: TNotifyEvent;
    FCanvas: TCanvas;
  protected
    procedure WMPaint(var Msg: TWMPaint); message WM_PAINT;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Canvas: TCanvas read FCanvas;
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    property OnKeyDown;
    property OnKeyUp;
    property OnKeyPress;
  end;

  TProgressEvent = procedure(Sender: TObject; Progress: Integer) of object;

  TGraphForm = class(TForm)
    GraphPanel: TPanel;
    AxisImage: TImage;
    ToolbarPanel: TPanel;
    BitrateThresholdEdit: TLabeledEdit;
    ApplyButton: TButton;
    StatusBar: TStatusBar;
    GotoEdit: TLabeledEdit;
    GoButton: TButton;
    FpsEdit: TLabeledEdit;
    ColorsButton: TButton;
    procedure GraphPanelResize(Sender: TObject);
    procedure PaintScrollBoxPaint(Sender: TObject);
    procedure ApplyButtonClick(Sender: TObject);
    procedure PaintScrollBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintScrollBoxMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure PaintScrollBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BitrateThresholdEditEnter(Sender: TObject);
    procedure GotoEditEnter(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure ColorsButtonClick(Sender: TObject);
  private
    FSelStart: Integer;
    FSelEnd: Integer;
    FOnSelectionChange: TNotifyEvent;
    FOnPositionChange: TNotifyEvent;
    FPosition: Integer;
    FOnLoadProgress: TProgressEvent;
    FDefaultButton: TButton;
    FFps: Extended;

    function GetFrames(i: Integer): TFrameInfo;
    function GetFrameCount: Integer;
    procedure SetBitrateThreshold(Value: Integer);
    procedure SetSelEnd(Value: Integer);
    procedure SetSelStart(Value: Integer);
    procedure InvertSelection(DC: HDC);
    procedure SetPosition(Value: Integer);
    procedure SetPositionSel(Value: Integer);
    procedure InvertPosition(DC: HDC);
    procedure UpdateStatusBar(Value: Integer);
    procedure SetDefaultButton(Value: TButton);
    procedure UpdateAverageBitrate;
    procedure ScrollView;
    procedure PaintScrollBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SelectionChange;
    procedure PositionChange;
  private
    { Private declarations }
    FPaintScrollBox: TPaintScrollBox;

    FFrames: TObjectList;
    FGraphWidth: Integer;
    FGraphHeight: Integer;
    FMaxFrameSize: Integer;
    FMaxFrameIndex: Integer;
    FTotalFrameSize: Integer;
    FAverageFrameSize: Integer;
    FBitrateThreshold: Integer;

    FColors: TGraphColors;

    procedure AddFrame(fi: TFrameInfo);
    procedure UpdateGraph;
    function SizeY(s: Integer): Integer;

    property FrameCount: Integer read GetFrameCount;
    property Frames[i: Integer]: TFrameInfo read GetFrames;

    property BitrateThreshold: Integer read FBitrateThreshold write SetBitrateThreshold;

    property DefaultButton: TButton read FDefaultButton write SetDefaultButton;
  public
    { Public declarations }

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property OnLoadProgress: TProgressEvent read FOnLoadProgress write FOnLoadProgress;
    procedure LoadFile(Filename: string);

    property MaxFrameIndex: Integer read FMaxFrameIndex;

    property SelStart: Integer read FSelStart write SetSelStart;
    property SelEnd: Integer read FSelEnd write SetSelEnd;
    property OnSelectionChange: TNotifyEvent read FOnSelectionChange write FOnSelectionChange;

    property Position: Integer read FPosition write SetPosition;
    property OnPositionChange: TNotifyEvent read FOnPositionChange write FOnPositionChange; 

    procedure SaveSettings(Registry: TRegistry);
    procedure LoadSettings(Registry: TRegistry);
  end;

var
  GraphForm: TGraphForm;

implementation

uses Math;

{$R *.dfm}

function NormDist(x, mu, sigma: Extended): Extended;
begin
  Result := exp(-(sqr(x - mu) / (2 * sqr(sigma)) )) / (sqrt(2 * Pi) * sigma);
end;

{ TGraphForm }

procedure TGraphForm.LoadFile(Filename: string);
const sigma = 5;
//const sigma = 1;

var f: TextFile;
  frameInfo: TFrameInfo;
  i, j, pct, oldPct: Integer;
  sum, denom: Extended;
begin
  FileMode := fmOpenRead or fmShareDenyNone;
  AssignFile(f, Filename);
  Reset(f);
  try
    ReadLn(f); // banner
    ReadLn(f); // banner
    ReadLn(f); // empty line

    FMaxFrameSize := 0;
    FFrames.Clear;
    FTotalFrameSize := 0;
    while not EOF(f) do
    begin
      frameInfo := TFrameInfo.Create;
      frameInfo.Read(f);
      AddFrame(frameInfo);
      Inc(FTotalFrameSize, frameInfo.FrameSize);
      if frameInfo.FrameSize > FMaxFrameSize then
      begin
        FMaxFrameSize := frameInfo.FrameSize;
        FMaxFrameIndex := FFrames.Count - 1;
      end;
    end;

    FAverageFrameSize := FTotalFrameSize div FrameCount;
    BitrateThreshold := Round(1.5 * FTotalFrameSize / FrameCount);
    UpdateAverageBitrate;

    oldPct := -1;
    for i := 0 to FrameCount - 1 do
    begin
      if Assigned(FOnLoadProgress) then
      begin
        pct := i * 100 div FrameCount;
        if oldPct <> pct then
        begin
          FOnLoadProgress(Self, pct);
          oldPct := pct;
        end;
      end;

      sum := 0;
      denom := 0;
      for j := Max(-3*sigma, -i) to Min(3*sigma, (FrameCount - 1) - i) do
      begin
        sum := sum + NormDist(j, 0, sigma) * Frames[i + j].FrameSize;
        denom := denom + NormDist(j, 0, sigma);
      end;
      Frames[i].FilteredFrameSize := Round(sum / denom);
    end;

    if Assigned(FOnLoadProgress) then
    begin
      FOnLoadProgress(Self, 100);
    end;

    UpdateGraph;
  finally
    CloseFile(f);
  end;
  Position := 0;
  UpdateStatusBar(0);
end;

procedure TGraphForm.AddFrame(fi: TFrameInfo);
begin
  FFrames.Add(fi);
end;

function TGraphForm.SizeY(s: Integer): Integer;
begin
  Result := s * FGraphHeight div FMaxFrameSize;
end;

function LogRoundUp(n: Extended): Integer;
begin
  case Ceil(n) of
    1: Result := 1;
    2: Result := 2;
    3..5: Result := 5;
    6..10: Result := 10;
    11..20: Result := 20;
  else
    Result := Trunc(IntPower(10, Trunc(Log10(n - 1)) + 1));
    case Trunc(Result / n) of
      1: ; // do nothing
      2..4: Result := Result div 2;
      5..9: Result := Result div 5;
    else
      Result := 0; // should not happen, will trigger a zero divide
    end
  end;
end;

procedure TGraphForm.UpdateGraph;
var i, right, bottom, top, y, TickUnit: Integer;
begin
  if not Assigned(AxisImage.Picture.Bitmap) then
    AxisImage.Picture.Bitmap := TBitmap.Create;

  AxisImage.Picture.Bitmap.Width := AxisImage.Width;
  AxisImage.Picture.Bitmap.Height := AxisImage.Height;

  FPaintScrollBox.HorzScrollBar.Range := FFrames.Count;

  FGraphWidth := FFrames.Count;
  FGraphHeight := AxisImage.Height - 100;

  with AxisImage.Picture.Bitmap, Canvas do
  begin
    right := Width - 1;
    bottom := Height - 50;
    top := 50;

    // Background
    Brush.Color := FColors.Background;
    Brush.Style := bsSolid;
    FillRect(ClipRect);

    // Axis
    Pen.Color := FColors.Axes;
    Pen.Style := psSolid;
    Pen.Width := 1;
    MoveTo(right, bottom);
    LineTo(right, top);

    // Ticks
    TickUnit := LogRoundUp(FMaxFrameSize * 32.0 / (bottom - top));

    Font.Name := 'MS Shell Dlg';
    Font.Size := 10;
    Font.Color := FColors.Text;
    SetTextAlign(Handle, TA_RIGHT or TA_BOTTOM);
    for i := 0 to (FMaxFrameSize - 1) div TickUnit do
    begin
      y := bottom - SizeY(i * TickUnit);
      MoveTo(right, y);
      LineTo(right - 2, y);
      TextOut(right - 5, y, IntToStr(i * TickUnit));
    end;
  end;

  FPaintScrollBox.Invalidate;
end;

function TGraphForm.GetFrames(i: Integer): TFrameInfo;
begin
  Result := FFrames[i] as TFrameInfo;
end;

constructor TGraphForm.Create(AOwner: TComponent);
begin
  inherited;
  FFrames := TObjectList.Create;
  FMaxFrameSize := 1;
  FFps := 23.976;
  FColors := TGraphColors.Create;

  FPaintScrollBox := TPaintScrollBox.Create(Self);
  FPaintScrollBox.Parent := GraphPanel;
  FPaintScrollBox.Align := alClient;
  FPaintScrollBox.Visible := True;
  FPaintScrollBox.BorderStyle := bsNone;
  FPaintScrollBox.TabStop := True;
  FPaintScrollBox.HorzScrollBar.Smooth := True;
  FPaintScrollBox.HorzScrollBar.Tracking := True;
  FPaintScrollBox.OnPaint := PaintScrollBoxPaint;
  FPaintScrollBox.OnMouseDown := PaintScrollBoxMouseDown;
  FPaintScrollBox.OnMouseMove := PaintScrollBoxMouseMove;
  FPaintScrollBox.OnMouseUp := PaintScrollBoxMouseUp;
  FPaintScrollBox.OnKeyDown := PaintScrollBoxKeyDown;
end;

destructor TGraphForm.Destroy;
begin
  FreeAndNil(FFrames);
  FreeAndNil(FColors);
  inherited;
end;

function TGraphForm.GetFrameCount: Integer;
begin
  Result := FFrames.Count;
end;

procedure TGraphForm.SetBitrateThreshold(Value: Integer);
begin
  FBitrateThreshold := Value;
  BitrateThresholdEdit.Text := IntToStr(Value);
end;

procedure TGraphForm.GraphPanelResize(Sender: TObject);
begin
  UpdateGraph;
end;

procedure TGraphForm.PaintScrollBoxPaint(Sender: TObject);
var top, bottom, left, right, width, i, x, y: Integer;
begin
  bottom := AxisImage.Height - 50;
  top := 50;
  left := FPaintScrollBox.HorzScrollBar.Position;
  width := FPaintScrollBox.Width;
  right := left + width;

  with FPaintScrollBox.Canvas do
  begin
    // Clear
    Brush.Color := FColors.Background;
    Brush.Style := bsSolid;
    FillRect(Rect(0, 0, FPaintScrollBox.Width, FPaintScrollBox.Height));

    // Axis
    Pen.Color := FColors.Axes;
    Pen.Style := psSolid;
    Pen.Width := 1;
    MoveTo(0, bottom);
    LineTo(width, bottom);

    // Size gridlines
    Pen.Color := FColors.Gridlines;
    for i := 0 to (FMaxFrameSize - 1) div 10000 do
    begin
      y := bottom - SizeY(i * 10000);
      MoveTo(0, y);
      LineTo(width, y);
    end;

    // Frame# gridlines and ticks
    Pen.Color := FColors.Gridlines;
    Font.Name := 'MS Shell Dlg';
    Font.Size := 10;
    Font.Color := FColors.Text;
    SetTextAlign(Handle, TA_LEFT or TA_TOP);
    for i := left div 100
    to (right - 1) div 100 do
    begin
      x := 100 * i;
      MoveTo(x - left, bottom - 1);
      LineTo(x - left, top);
      TextOut(x - left, bottom + 5, IntToStr(x));
    end;

    // Data
    for i := left to right - 1 do
    begin
      Pen.Color := FColors.Filtered;
      MoveTo(i - left, bottom);
      LineTo(i - left, bottom - SizeY(Frames[i].FilteredFrameSize));

      Pen.Color := FColors.Data;
      MoveTo(i - left, bottom);
      LineTo(i - left, bottom - SizeY(Frames[i].FrameSize));
    end;

    // Threshold line
    Pen.Color := FColors.Threshold;
    y := bottom - SizeY(FBitrateThreshold);
    MoveTo(0, y);
    LineTo(width, y);

    // Average line
    Pen.Color := FColors.Average;
    y := bottom - SizeY(FAverageFrameSize);
    MoveTo(0, y);
    LineTo(width, y);

    // Selection
    InvertRect(Handle, Rect(FSelStart - left, top, FSelEnd - left, bottom));

    // Position
    InvertRect(Handle, Rect(FPosition - left, top, FPosition + 1 - left, bottom));
  end;
end;

procedure TGraphForm.ApplyButtonClick(Sender: TObject);
begin
  FBitrateThreshold := StrToInt(BitrateThresholdEdit.Text);
  FFps := StrToFloat(FpsEdit.Text);
  UpdateGraph;
  UpdateAverageBitrate;
end;

procedure TGraphForm.SetSelEnd(Value: Integer);
begin
  Value := Max(0, Min(FrameCount, Value));

  FSelEnd := Value;
  FPaintScrollBox.Invalidate;
end;

procedure TGraphForm.SetSelStart(Value: Integer);
begin
  Value := Max(0, Min(FrameCount - 1, Value));

  FSelStart := Value;
  FPaintScrollBox.Invalidate;
end;

procedure TGraphForm.InvertPosition(DC: HDC);
begin
  InvertRect(DC, Rect(
    FPosition - FPaintScrollBox.HorzScrollBar.Position,
    AxisImage.Height - 50 - SizeY(FMaxFrameSize),
    FPosition + 1 - FPaintScrollBox.HorzScrollBar.Position,
    AxisImage.Height - 50));
end;

procedure TGraphForm.SetPosition(Value: Integer);
var DC: HDC;
begin
  Value := Max(0, Min(FrameCount - 1, Value));
  if Value = FPosition then Exit;

  DC := GetDC(FPaintScrollBox.Handle);
  try
    InvertPosition(DC);
    FPosition := Value;
    InvertPosition(DC);
  finally
    ReleaseDC(FPaintScrollBox.Handle, DC);
  end;

  ScrollView;
  UpdateStatusBar(Value);
  PositionChange;
end;

procedure TGraphForm.ScrollView;
begin
  if (FPosition < FPaintScrollBox.HorzScrollBar.Position) or
    (FPosition > FPaintScrollBox.HorzScrollBar.Position + FPaintScrollBox.Width) then
  begin
    FPaintScrollBox.HorzScrollBar.Position :=
      Max(0, Min(FrameCount - FPaintScrollBox.Width,
      FPosition - FPaintScrollBox.Width div 2));
  end;
end;

procedure TGraphForm.UpdateStatusBar(Value: Integer);
begin
  StatusBar.Panels[0].Text := IntToStr(Value);
  with Frames[Value] do
  begin
    StatusBar.Panels[1].Text := UpperCase(FrameType);
    StatusBar.Panels[2].Text := IntToStr(Quantizer);
    StatusBar.Panels[3].Text := IntToStr(IntraBlocks);
    StatusBar.Panels[4].Text := IntToStr(InterBlocks);
    StatusBar.Panels[5].Text := IntToStr(NotCodedBlocks);
    StatusBar.Panels[6].Text := IntToStr(FrameSize);
    StatusBar.Panels[7].Text := IntToStr(HeaderSize);
  end;
end;

procedure TGraphForm.SetDefaultButton(Value: TButton);
begin
  if Assigned(FDefaultButton) then
  begin
    FDefaultButton.Default := False;
  end;
  FDefaultButton := Value;
  if Assigned(FDefaultButton) then
  begin
    FDefaultButton.Default := True;
  end;
end;

procedure TGraphForm.SetPositionSel(Value: Integer);
var DC: HDC;
begin
  Value := Max(0, Min(FrameCount - 1, Value));

  DC := GetDC(FPaintScrollBox.Handle);
  try
    InvertPosition(DC);

    FPosition := Value;
    FSelStart := Value;
    FSelEnd := Value;

    InvertPosition(DC);
  finally
    ReleaseDC(FPaintScrollBox.Handle, DC);
  end;

  UpdateStatusBar(Value);
  PositionChange;
end;

procedure TGraphForm.PositionChange;
begin
  if Assigned(FOnPositionChange) then
  begin
    FOnPositionChange(Self);
  end;
end;

{ TFrameInfo }

procedure TFrameInfo.Read(var f: TextFile);
begin
  ReadLn(f, FFrameType, FQuantizer,
    FIntraBlocks, FInterBlocks, FNotCodedBlocks,
    FFrameSize, FHeaderSize);
end;

procedure TGraphForm.PaintScrollBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    FSelStart := X + FPaintScrollBox.HorzScrollBar.Position;
    SelEnd := FSelStart;
  end;
end;

procedure TGraphForm.InvertSelection(DC: HDC);
begin
  InvertRect(DC, Rect(
    FSelStart - FPaintScrollBox.HorzScrollBar.Position,
    AxisImage.Height - 50 - SizeY(FMaxFrameSize),
    FSelEnd - FPaintScrollBox.HorzScrollBar.Position,
    AxisImage.Height - 50));
end;

procedure TGraphForm.PaintScrollBoxMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var DC: HDC;
begin
  if ssLeft in Shift then
  begin
    DC := GetDC(FPaintScrollBox.Handle);
    try
      InvertSelection(DC); // Clear selection
      FSelEnd := X + FPaintScrollBox.HorzScrollBar.Position;
      UpdateStatusBar(FSelEnd);
      InvertSelection(DC); // Draw selection
    finally
      ReleaseDC(FPaintScrollBox.Handle, DC);
    end;
  end;
end;

procedure TGraphForm.PaintScrollBoxMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var DC: HDC;
begin
  if Button = mbLeft then
  begin
    DC := GetDC(FPaintScrollBox.Handle);
    try
      InvertSelection(DC); // Clear selection
    finally
      ReleaseDC(FPaintScrollBox.Handle, DC);
    end;
    SelEnd := X + FPaintScrollBox.HorzScrollBar.Position;
    Position := FSelStart;

    SelectionChange;
  end;
end;

procedure TGraphForm.SaveSettings(Registry: TRegistry);
begin
  try Registry.WriteInteger('Left', Left); except end;
  try Registry.WriteInteger('Top', Top); except end;
  try Registry.WriteInteger('Width', Width); except end;
  try Registry.WriteInteger('Height', Height); except end;
  try Registry.WriteFloat('FPS', FFps); except end;
  FColors.SaveToRegistry(Registry);
end;

procedure TGraphForm.LoadSettings(Registry: TRegistry);
begin
  try Left := Registry.ReadInteger('Left'); except end;
  try Top := Registry.ReadInteger('Top'); except end;
  try Width := Registry.ReadInteger('Width'); except end;
  try Height := Registry.ReadInteger('Height'); except end;
  try FFps := Registry.ReadFloat('FPS'); except end;
  FColors.LoadFromRegistry(Registry);
end;

procedure TGraphForm.BitrateThresholdEditEnter(Sender: TObject);
begin
  DefaultButton := ApplyButton;
end;

procedure TGraphForm.GotoEditEnter(Sender: TObject);
begin
  DefaultButton := GoButton;
end;

procedure TGraphForm.GoButtonClick(Sender: TObject);
var Value: Integer;
begin
  Value := Max(0, Min(FrameCount - 1, StrToInt(GotoEdit.Text)));
  SetPositionSel(Value);
  SelectionChange;
end;

procedure TGraphForm.UpdateAverageBitrate;
begin
  StatusBar.Panels[8].Text := Format('Average bitrate: %f kbps', [
    FFps{frames/s} * FTotalFrameSize{byte} * 8{bit/byte} /
    (FrameCount{frames} * 1000{bit/kbit})]);
end;

procedure TGraphForm.ColorsButtonClick(Sender: TObject);
begin
  with TColorsForm.Create(Self) do
  try
    Colors.Assign(FColors);
    if Execute then
    begin
      FColors.Assign(Colors);
      UpdateGraph;
    end;
  finally
    Free;
  end;
end;

procedure TGraphForm.PaintScrollBoxKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);

  const
    VK_OEM_4 = $DB;
    VK_OEM_6 = $DD;

begin
  case Key of
    VK_LEFT:
      if [ssCtrl] = Shift then
      begin
        Position := 0;
        Key := 0;
      end
      else if [ssShift] = Shift then
      begin
        Position := Position - 10;
        Key := 0;
      end
      else if [] = Shift then
      begin
        Position := Position - 1;
        Key := 0;
      end;
    VK_RIGHT:
      if [ssCtrl] = Shift then
      begin
        Position := FrameCount - 1;
        Key := 0;
      end
      else if [ssShift] = Shift then
      begin
        Position := Position + 10;
        Key := 0;
      end
      else if [] = Shift then
      begin
        Position := Position + 1;
        Key := 0;
      end;
    VK_PRIOR:
      if [ssCtrl] = Shift then
      begin
        Position := FPaintScrollBox.HorzScrollBar.Position;
      end
      else if [] = Shift then
      begin
        FPaintScrollBox.HorzScrollBar.Position := FPaintScrollBox.HorzScrollBar.Position - FPaintScrollBox.Width;
        Position := Position - FPaintScrollBox.Width;
      end;
    VK_NEXT:
      if [ssCtrl] = Shift then
      begin
        Position := FPaintScrollBox.HorzScrollBar.Position + FPaintScrollBox.Width - 1;
      end
      else if [] = Shift then
      begin
        FPaintScrollBox.HorzScrollBar.Position := FPaintScrollBox.HorzScrollBar.Position + FPaintScrollBox.Width;
        Position := Position + FPaintScrollBox.Width;
      end;
    VK_OEM_4: // [
      begin
        Position := SelStart;
      end;
    VK_OEM_6: // ]
      begin
        Position := SelEnd;
      end;
    VK_HOME:
      begin
        SelStart := Position;
        SelectionChange;
      end;
    VK_END:
      begin
        SelEnd := Position;
        SelectionChange;
      end;
  end;
end;

procedure TGraphForm.SelectionChange;
begin
  if Assigned(FOnSelectionChange) then
  begin
    FOnSelectionChange(Self);
  end;
end;

{ TPaintScrollBox }

constructor TPaintScrollBox.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle - [csAcceptsControls];
end;

procedure TPaintScrollBox.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure TPaintScrollBox.WMPaint(var Msg: TWMPaint);
begin
  inherited;
  FCanvas := TCanvas.Create;
  try
    FCanvas.Handle := GetDC(Handle);
    try
      if Assigned(FOnPaint) then
      begin
        FOnPaint(Self);
      end;
    finally
      ReleaseDC(Handle, FCanvas.Handle);
    end;
  finally
    FCanvas.Free;
  end;
end;

end.
