{
    XviD Stats Analyzer
    Copyright (C) 2005 Yuri Khan
    stats-analyzer at gorodok dot net

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

unit StatsAnalyzerUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, Registry, ComCtrls;

type
  TStatsAnalyzerForm = class(TForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    FileOpenStatsCommand: TMenuItem;
    FileOpenVideoCommand: TMenuItem;
    FileSeparator1: TMenuItem;
    FileExitCommand: TMenuItem;
    OpenDialog: TOpenDialog;
    OpenVideoDialog: TOpenDialog;
    StatsMemo: TMemo;
    ViewMenu: TMenuItem;
    ViewVideoCommand: TMenuItem;
    ViewGraphCommand: TMenuItem;
    FileSeparator2: TMenuItem;
    StatusBar: TStatusBar;
    ViewSeparator1: TMenuItem;
    ViewFontCommand: TMenuItem;
    FontDialog: TFontDialog;
    HelpMenu: TMenuItem;
    HelpAboutCommand: TMenuItem;
    procedure FileExitCommandClick(Sender: TObject);
    procedure FileOpenStatsCommandClick(Sender: TObject);
    procedure ViewGraphCommandClick(Sender: TObject);
    procedure ViewVideoCommandClick(Sender: TObject);
    procedure FileOpenVideoCommandClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure StatusBarDrawPanel(StatusBar: TStatusBar;
      Panel: TStatusPanel; const Rect: TRect);
    procedure ViewFontCommandClick(Sender: TObject);
    procedure FontDialogApply(Sender: TObject; Wnd: HWND);
    procedure HelpAboutCommandClick(Sender: TObject);
  private
    { Private declarations }

    FMaxRecentFiles: Integer;
    FRecentFiles: TStringList;

    FLoading: Boolean;
    FLoadProgress: Integer;

    FStatsFilename: string;
    FVideoFilename: string;
    FAutoLoadStats: Boolean;

    procedure LoadStatsFile(Filename: string);
    procedure GraphFormSelectionChange(Sender: TObject);
    procedure VideoPlayerFormPositionChange(Sender: TObject);
    procedure GraphFormShow(Sender: TObject);
    procedure VideoPlayerFormShow(Sender: TObject);
    procedure LoadAllSettings;
    procedure LoadSettings(Registry: TRegistry);
    procedure SaveAllSettings;
    procedure SaveSettings(Registry: TRegistry);
    procedure LoadVideoFile(Filename: string);
    procedure UpdateRecentFiles;
    procedure FileRecentCommandClick(Sender: TObject);
    function StatsFilename(VideoFilename: string): string;
    procedure GraphFormLoadProgress(Sender: TObject; Progress: Integer);
    procedure LoadFile(Filename: string);
    function IsStatsFilename(Filename: string): Boolean;
    function IsVideoFilename(Filename: string): Boolean;
    procedure AddRecentFile(Filename: string);
    procedure GraphFormPositionChange(Sender: TObject);
  public
    { Public declarations }

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  StatsAnalyzerForm: TStatsAnalyzerForm;

implementation

uses GraphUnit, VideoPlayerUnit, Types, AboutUnit;

{$R *.dfm}

procedure TStatsAnalyzerForm.FileExitCommandClick(Sender: TObject);
begin
  Close;
end;

procedure TStatsAnalyzerForm.AddRecentFile(Filename: string);
begin
  Filename := ExpandUNCFileName(Filename);
  if FRecentFiles.IndexOf(Filename) = -1 then
  begin
    FRecentFiles.Insert(0, Filename);
    UpdateRecentFiles;
  end
end;

procedure TStatsAnalyzerForm.FileOpenStatsCommandClick(Sender: TObject);
begin
  if OpenDialog.Execute then
  begin
    LoadStatsFile(OpenDialog.FileName);
    AddRecentFile(OpenDialog.FileName);
  end;
end;

procedure TStatsAnalyzerForm.FileOpenVideoCommandClick(Sender: TObject);
begin
  if OpenVideoDialog.Execute then
  begin
    LoadVideoFile(OpenVideoDialog.FileName);
    AddRecentFile(OpenVideoDialog.FileName);
    LoadStatsFile(StatsFilename(OpenVideoDialog.FileName));
  end;
end;

procedure TStatsAnalyzerForm.LoadStatsFile(Filename: string);
begin
  if Filename = '' then Exit;

  StatsMemo.Lines.LoadFromFile(Filename);
  FLoading := True;
  try
    GraphForm.LoadFile(Filename);
  finally
    FLoading := False;
    StatusBar.Invalidate;
  end;
  GraphForm.Show;

  FStatsFilename := Filename;
end;

procedure TStatsAnalyzerForm.ViewGraphCommandClick(Sender: TObject);
begin
  ViewGraphCommand.Checked := not ViewGraphCommand.Checked;
  GraphForm.Visible := ViewGraphCommand.Checked;
end;

procedure TStatsAnalyzerForm.ViewVideoCommandClick(Sender: TObject);
begin
  ViewVideoCommand.Checked := not ViewVideoCommand.Checked;
  VideoPlayerForm.Visible := ViewVideoCommand.Checked;
end;

procedure TStatsAnalyzerForm.GraphFormSelectionChange(Sender: TObject);
begin
  StatsMemo.SelStart := StatsMemo.Perform(EM_LINEINDEX, 3 + GraphForm.SelStart, 0);
  StatsMemo.SelLength := StatsMemo.Perform(EM_LINEINDEX, 3 + GraphForm.SelEnd, 0) - StatsMemo.SelStart;
  StatsMemo.Perform(EM_SCROLLCARET, 0, 0);

  VideoPlayerForm.Pause;
  VideoPlayerForm.SelStart := GraphForm.SelStart;
  VideoPlayerForm.SelEnd := GraphForm.SelEnd;
end;

function TStatsAnalyzerForm.StatsFilename(VideoFilename: string): string;
begin
  Result := ChangeFileExt(VideoFilename, '.pass');
  if FileExists(Result) then Exit;

  Result := ChangeFileExt(VideoFilename, '.stats');
  if FileExists(Result) then Exit;

  Result := ExtractFilePath(VideoFileName) + '..\stats\' +
    ChangeFileExt(ExtractFileName(VideoFileName), '.pass');
  if FileExists(Result) then Exit;

  Result := ExtractFilePath(VideoFileName) + '..\stats\' +
    ChangeFileExt(ExtractFileName(VideoFileName), '.stats');
  if FileExists(Result) then Exit;

  Result := '';
end;

procedure TStatsAnalyzerForm.LoadVideoFile(Filename: string);
begin
  VideoPlayerForm.LoadFile(Filename);
  VideoPlayerForm.Show;
  FVideoFilename := Filename;
end;

procedure TStatsAnalyzerForm.VideoPlayerFormPositionChange(Sender: TObject);
begin
  GraphForm.Position := VideoPlayerForm.Position;
end;

procedure TStatsAnalyzerForm.GraphFormPositionChange(Sender: TObject);
begin
  VideoPlayerForm.Position := GraphForm.Position;
end;

constructor TStatsAnalyzerForm.Create(AOwner: TComponent);
var i: Integer; errors: string;
begin
  inherited;

  GraphForm := TGraphForm.Create(Self);
  GraphForm.OnSelectionChange := GraphFormSelectionChange;
  GraphForm.OnPositionChange := GraphFormPositionChange;
  GraphForm.OnShow := GraphFormShow;
  GraphForm.OnHide := GraphFormShow;
  GraphForm.OnLoadProgress := GraphFormLoadProgress;

  VideoPlayerForm := TVideoPlayerForm.Create(Self);
  VideoPlayerForm.OnPositionChange := VideoPlayerFormPositionChange;
  VideoPlayerForm.OnShow := VideoPlayerFormShow;
  VideoPlayerForm.OnHide := VideoPlayerFormShow;

  FMaxRecentFiles := 4;
  FRecentFiles := TStringList.Create;

  LoadAllSettings;

  Show;

  FAutoLoadStats := False;
  errors := '';

  for i := 1 to ParamCount do
  begin
    if '/auto' = AnsiLowerCase(ParamStr(i)) then
    begin
      FAutoLoadStats := True;
    end
    else
    try
      LoadFile(ParamStr(i));
    except
      on E: Exception do
      begin
        errors := errors + E.Message + #13;
      end;
    end;
  end;

  if FAutoLoadStats and (Length(FVideoFilename) > 0) then
  try
    LoadStatsFile(StatsFilename(FVideoFilename));
  except
    on E: Exception do
    begin
      errors := errors + E.Message + #13;
    end;
  end;

  if Length(errors) > 0 then
  begin
    Application.MessageBox(PChar(errors), 'Error', MB_ICONERROR);
  end;
end;

function TStatsAnalyzerForm.IsStatsFilename(Filename: string): Boolean;
var ext: string;
begin
  ext := AnsiLowerCase(ExtractFileExt(Filename));
  Result := (ext = '.pass') or (ext = '.stats');
end;

function TStatsAnalyzerForm.IsVideoFilename(Filename: string): Boolean;
begin
  Result := AnsiLowerCase(ExtractFileExt(Filename)) = '.avi';
end;

procedure TStatsAnalyzerForm.LoadFile(Filename: string);
begin
  if IsStatsFilename(Filename) then
  begin
    LoadStatsFile(Filename);
    AddRecentFile(Filename);
  end
  else if IsVideoFilename(Filename) then
  begin
    LoadVideoFile(Filename);
    AddRecentFile(Filename);
  end
  else
  begin
    raise Exception.CreateFmt('Unknown filename extension: %s', [Filename]); 
  end;
end;

procedure TStatsAnalyzerForm.GraphFormLoadProgress(Sender: TObject; Progress: Integer);
begin
  FLoadProgress := Progress;
  StatusBar.Invalidate;
  Application.ProcessMessages;
end;

procedure TStatsAnalyzerForm.UpdateRecentFiles;
var i, j: Integer; item: TMenuItem;
begin
  while FRecentFiles.Count > FMaxRecentFiles do
  begin
    FRecentFiles.Delete(FRecentFiles.Count - 1);
  end;

  i := FileMenu.IndexOf(FileSeparator1) + 1;
  while FileMenu[i] <> FileSeparator2 do
  begin
    item := FileMenu[i];
    FileMenu.Delete(i);
    FreeAndNil(item);
  end;

  for j := 0 to FRecentFiles.Count - 1 do
  begin
    item := TMenuItem.Create(Self);
    case j of
    0..8:
      item.Caption := Format('&%d %s', [j + 1, FRecentFiles[j]]);
    9:
      item.Caption := Format('1&0 %s', [FRecentFiles[j]]);
    else
      item.Caption := Format('%d %s', [j + 1, FRecentFiles[j]]);
    end;
    item.Tag := j;
    item.OnClick := FileRecentCommandClick;
    FileMenu.Insert(i, item);
    Inc(i);
  end;
end;

procedure TStatsAnalyzerForm.FileRecentCommandClick(Sender: TObject);
var Filename: string;
begin
  Filename := FRecentFiles[(Sender as TMenuItem).Tag];
  if IsStatsFilename(Filename) then
  begin
    LoadStatsFile(Filename);
  end
  else
  begin
    LoadVideoFile(Filename);
    LoadStatsFile(StatsFilename(Filename));
  end;
end;

procedure TStatsAnalyzerForm.GraphFormShow(Sender: TObject);
begin
  ViewGraphCommand.Checked := GraphForm.Visible;
end;

procedure TStatsAnalyzerForm.VideoPlayerFormShow(Sender: TObject);
begin
  ViewVideoCommand.Checked := VideoPlayerForm.Visible;
end;

procedure TStatsAnalyzerForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SaveAllSettings;
end;

procedure TStatsAnalyzerForm.SaveAllSettings;
var Registry: TRegistry;
begin
  Registry := TRegistry.Create(KEY_SET_VALUE or KEY_CREATE_SUB_KEY);
  try
    Registry.RootKey := HKEY_CURRENT_USER;
    Registry.OpenKey('\Software\Centaur\XvidStatsAnalyzer', True);
    try
      SaveSettings(Registry);
    finally
      Registry.CloseKey;
    end;

    Registry.OpenKey('\Software\Centaur\XvidStatsAnalyzer\Graph', True);
    try
      GraphForm.SaveSettings(Registry);
    finally
      Registry.CloseKey;
    end;

    Registry.OpenKey('\Software\Centaur\XvidStatsAnalyzer\Video', True);
    try
      VideoPlayerForm.SaveSettings(Registry);
    finally
      Registry.CloseKey;
    end;
  finally
    FreeAndNil(Registry);
  end;
end;

procedure TStatsAnalyzerForm.LoadAllSettings;
var Registry: TRegistry;
begin
  Registry := TRegistry.Create(KEY_QUERY_VALUE);
  try
    Registry.RootKey := HKEY_CURRENT_USER;
    if Registry.OpenKey('\Software\Centaur\XvidStatsAnalyzer', False) then
    try
      LoadSettings(Registry);
    finally
      Registry.CloseKey;
    end;

    if Registry.OpenKey('\Software\Centaur\XvidStatsAnalyzer\Graph', False) then
    try
      GraphForm.LoadSettings(Registry);
    finally
      Registry.CloseKey;
    end;

    if Registry.OpenKey('\Software\Centaur\XvidStatsAnalyzer\Video', False) then
    try
      VideoPlayerForm.LoadSettings(Registry);
    finally
      Registry.CloseKey;
    end;
  finally
    FreeAndNil(Registry);
  end;
end;

procedure TStatsAnalyzerForm.SaveSettings(Registry: TRegistry);
begin
  try Registry.WriteInteger('Left', Left); except end;
  try Registry.WriteInteger('Top', Top); except end;
  try Registry.WriteInteger('Width', Width); except end;
  try Registry.WriteInteger('Height', Height); except end;
  try Registry.WriteString('RecentFiles', FRecentFiles.CommaText); except end;
  try Registry.WriteString('Font.Name', StatsMemo.Font.Name); except end;
  try Registry.WriteInteger('Font.Charset', StatsMemo.Font.Charset); except end;
  try Registry.WriteString('Font.Color', ColorToString(StatsMemo.Font.Color)); except end;
  try Registry.WriteInteger('Font.Size', StatsMemo.Font.Size); except end;
end;

procedure TStatsAnalyzerForm.LoadSettings(Registry: TRegistry);
begin
  try Left := Registry.ReadInteger('Left'); except end;
  try Top := Registry.ReadInteger('Top'); except end;
  try Width := Registry.ReadInteger('Width'); except end;
  try Height := Registry.ReadInteger('Height'); except end;
  try FMaxRecentFiles := Registry.ReadInteger('MaxRecentFiles'); except end;
  try FRecentFiles.CommaText := Registry.ReadString('RecentFiles'); except end;
  try StatsMemo.Font.Name := Registry.ReadString('Font.Name'); except end;
  try StatsMemo.Font.Charset := Registry.ReadInteger('Font.Charset'); except end;
  try StatsMemo.Font.Color := StringToColor(Registry.ReadString('Font.Color')); except end;
  try StatsMemo.Font.Size := Registry.ReadInteger('Font.Size'); except end;
  UpdateRecentFiles;
end;

destructor TStatsAnalyzerForm.Destroy;
begin
  FreeAndNil(FRecentFiles);
  inherited;
end;

procedure TStatsAnalyzerForm.StatusBarDrawPanel(StatusBar: TStatusBar;
  Panel: TStatusPanel; const Rect: TRect);
begin
  if Panel.ID = 0 then
  begin
    if FLoading then
    begin
      StatusBar.Canvas.Brush.Color := clHighlight;
      StatusBar.Canvas.Brush.Style := bsSolid;
      StatusBar.Canvas.FillRect(Types.Rect(Rect.Left, Rect.Top,
        Rect.Left + (Rect.Right - Rect.Left) * FLoadProgress div 100,
        Rect.Bottom));
    end
    else
    begin
      StatusBar.Canvas.Brush.Color := clBtnFace;
      StatusBar.Canvas.Brush.Style := bsSolid;
      StatusBar.Canvas.FillRect(Rect);
    end;
  end;
end;

procedure TStatsAnalyzerForm.ViewFontCommandClick(Sender: TObject);
begin
  FontDialog.Font.Assign(StatsMemo.Font);
  if FontDialog.Execute then
  begin
    StatsMemo.Font.Assign(FontDialog.Font);
  end;
end;

procedure TStatsAnalyzerForm.FontDialogApply(Sender: TObject; Wnd: HWND);
begin
  StatsMemo.Font.Assign((Sender as TFontDialog).Font);
end;

procedure TStatsAnalyzerForm.HelpAboutCommandClick(Sender: TObject);
begin
  with TAboutForm.Create(Self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

end.
